/* License
 * 
 * Copyright 1994-2004 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *  
 *  * Redistribution of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 * 
 *  * Redistribution in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 * Neither the name of Sun Microsystems, Inc. or the names of contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *  
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING
 * ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN MICROSYSTEMS, INC. ("SUN")
 * AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE
 * AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
 * DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE LIABLE FOR ANY LOST
 * REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY
 * OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
 * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *  
 * You acknowledge that this software is not designed, licensed or intended
 * for use in the design, construction, operation or maintenance of any
 * nuclear facility. 
 */
package org.piratis.j2me.demos.tank;

import javax.microedition.lcdui.Image;

import org.piratis.j2me.core.game.Sprite;

public class MicroTankSprite
    extends Sprite {
  private int mDirection;
  private int mKX, mKY;
  
  private int mLastDelta;
  private boolean mLastWasTurn;
  
  
  private static final int[] kFrameLookup = {
    0, 1, 2, 3,
    4, 5, 6, 7,
    8, 9, 10, 11,
    12, 13, 14, 15
  };
  
  private static final int[] kCosLookup = {
       0,  383,  707,  924,
    1000,  924,  707,  383,
       0, -383, -707, -924,
   -1000, -924, -707, -383
  };

  private static final int[] kSinLookup = {
    1000,  924,  707,  383,
       0, -383, -707, -924,
   -1000, -924, -707, -383,
       0,  383,  707,  924
  };

  public MicroTankSprite(Image image, int frameWidth, int frameHeight) {
    super(image, frameWidth, frameHeight);
    mDirection = 0;
  }
  
  public void turn(int delta) {
    mDirection += delta;
    if (mDirection < 0) mDirection += 16;
    if (mDirection > 15) mDirection %= 16;
    
    setFrame(kFrameLookup[mDirection]);
    
    mLastDelta = delta;
    mLastWasTurn = true;
  }
  
  public void forward(int delta) {
    fineMove(kCosLookup[mDirection] * delta,
        -kSinLookup[mDirection] * delta);
    mLastDelta = delta;
    mLastWasTurn = false;
  }
  
  public void undo() {
    if (mLastWasTurn)
      turn(-mLastDelta);
    else
      forward(-mLastDelta);
  }
  
  private void fineMove(int kx, int ky) {
    // First initialize mKX and mKY if they're
    // not close enough to the actual x and y.
    int x = this.bbox.x;
    int y = this.bbox.y;
    int errorX = Math.abs(mKX - x * 1000);
    int errorY = Math.abs(mKY - y * 1000);
    if (errorX > 1000 || errorY > 1000) {
      mKX = x * 1000;
      mKY = y * 1000;
    }
    // Now add the deltas.
    mKX += kx;
    mKY += ky;
    // Set the actual position.
    this.moveTo(mKX / 1000, mKY / 1000);
  }
}
